<?php

include "Akrabat/Config/Abstract.php";

class Akrabat_Config_Ini extends Akrabat_Config_Abstract
{

    /**
     * Load a config file, overwriting any duplicate keys.
     *
     * @param string $filename
     * @param string $section
     */
    public function load($filename, $section)
    {
        $iniArray = parse_ini_file($filename, true);
        if(!isset($iniArray[$section])) {
            throw new Exception("Section '$section' cannot be found in $filename!");
        }

        $this->_config = $this->processExtends($iniArray, $section);


    }

    /**
     * Helper function to process each element in the section and handle
     * the "extends" inheritance keyword. Passes control to processKey()
     * to handle the "dot" sub-property syntax in each key.
     *
     * @param array $iniArray
     * @param string $section
     * @return array
     */
    protected function processExtends($iniArray, $section)
    {
        $thisSection = $iniArray[$section];
        $config = new stdClass();
        foreach ($thisSection as $key => $value) {
            if ($key == 'extends') {
                $sections = explode(',', $value);
                foreach ($sections as $s) {
                    if( isset($iniArray[$s])) {
                        foreach($iniArray[$s] as $k => $v) {
                            if ($k == 'extends') {
                                throw new Exception("May not extend a second time!");
                            }
                            $config=$this->processKey($config, $k, $v);
                        }
                    }
                    else
                    {
                        throw new Exception("Section '$section' cannot be found!");
                    }
                }
            }
            else
            {
                $config = $this->processKey($config, $key, $value);
            }
        }
        return $config;
    }

    /**
     * Assign the key's value to the property list. Handle the "dot"
     * notation for sub-properties by passing control to
     * processLevelsInKey().
     *
     * @param stdClass $config
     * @param string $key
     * @param string $value
     * @return stdClass
     */
    function processKey($config, $key , $value)
    {
        if (strpos($key, '.') !== false) {
            $pieces = explode('.', $key, 2);
            if (!empty($pieces[0]) && !empty($pieces[1])) {
                if(!isset($config->$pieces[0])) {
                    $config->$pieces[0] = new stdClass();
                }
                $config->$pieces[0] = $this->processLevelsInKey($config->$pieces[0], $pieces[1], $value);
            }
            else {
                throw new Exception ("Invalid key '$key'");
            }
        } else {
            $config->$key = $value;
        }
        return $config;
    }


    /**
     * Helper function to handle the "dot" namespace syntax in the key.
     * Uses "." as the separator.
     *
     * @param stdClass $parent
     * @param string $key
     * @param string $value
     * @return stdClass
     */
    protected function processLevelsInKey($parent, $key, $value)
    {
        if (strpos($key, '.')) {
            $pieces = explode('.', $key, 2);
            if (!empty($pieces[0]) && !empty($pieces[1])) {
                if(!isset($parent->$pieces[0])) {
                    $parent->$pieces[0] = new stdClass();
                }
                elseif(!is_object($parent->$pieces[0])) {
                    throw new Exception("cannot create sub-key for '$key' as key is already created.");
                }
                $parent->$pieces[0] = $this->processLevelsInKey($parent->$pieces[0], $pieces[1], $value);
            } else {
                $parent->$key = $value;
            }
        } else {
            if(!empty($key))
            {
                $parent->$key = $value;
            }
        }
        return $parent;
    }

}
?>