<?php
require_once 'PHPUnit2/Framework/TestCase.php';
include_once "Zend.php";

class Config_IniTest extends PHPUnit2_Framework_TestCase
{
    protected $_iniFileConfig;
    protected $_iniFileNested;

    function setUp()
    {
        Zend::loadClass('Akrabat_Config_Ini');
        $this->_iniFileConfig = dirname(__FILE__).'/_files/config.ini';
    }

    function tearDown()
    {
    }

    function testArray()
    {
        $config = new Akrabat_Config_Ini($this->_iniFileConfig, 'all');
        ob_start();
        print_r((array)$config->db);
        $contents = ob_get_contents();
        ob_end_clean();

        $this->assertContains('Array', $contents);
        $this->assertContains('[user] => username', $contents);
    }

    function testIterator()
    {
        $config = new Akrabat_Config_Ini($this->_iniFileConfig, 'all');

        $var = '';
        foreach ($config as $key=>$value) {
            $var .= "\nkey = $key, value = $value";
        }
        $this->assertContains('key = name, value = thisname', $var);
    }

    function testLoadSingleSection()
    {
        $config = new Akrabat_Config_Ini($this->_iniFileConfig, 'all');
        $this->assertEquals('all', $config->hostname);
        $this->assertEquals('live', $config->db->name);
        $this->assertEquals('multi', $config->one->two->three);
        $this->assertNull($config->nonexistant); // property doesn't exist
    }

    function testSectionInclude()
    {
        $config = new Akrabat_Config_Ini($this->_iniFileConfig, 'staging');
        $this->assertEquals('', $config->debug); // only in staging
        $this->assertEquals('thisname', $config->name); // only in all
        $this->assertEquals('staging', $config->hostname); // inherited
        $this->assertEquals('dbstaging', $config->db->name); //inherited
    }

    function testSectionNoNestingOfExtends()
    {
        try {
            $config = new Akrabat_Config_Ini($this->_iniFileConfig, 'doubleExtend');
        }
        catch (Exception $expected) {
            $this->assertEquals('May not extend a second time!', $expected->getMessage());
            return;
        }
    }

    function testSectionMultiExtends()
    {
        $config = new Akrabat_Config_Ini($this->_iniFileConfig, 'debug');
        $this->assertEquals('1', $config->debug); // only in staging
        $this->assertEquals('thisname', $config->name); // only in all
        $this->assertEquals('debug', $config->hostname); // inherited
        $this->assertEquals('dbdebug', $config->db->name); //inherited
    }

    function testModification()
    {
        $config = new Akrabat_Config_Ini($this->_iniFileConfig, 'all', true);

        // overwrite an existing key
        $this->assertEquals('thisname', $config->name);
        $config->name = 'anothername';
        $this->assertEquals('anothername', $config->name);

        // overwrite an existing multi-level key
        $this->assertEquals('multi', $config->one->two->three);
        $config->one->two->three = 'anothername';
        $this->assertEquals('anothername', $config->one->two->three);

        // create a new multi-level key
        $config->does = new stdClass();
        $config->does->not = new stdClass();
        $config->does->not->exist = 'yet';
        $this->assertEquals('yet', $config->does->not->exist);


        // re-load file and check that file hasn't changed
        $another_config = new Akrabat_Config_Ini($this->_iniFileConfig, 'all', true);
        $this->assertEquals('thisname', $another_config->name);
        $this->assertEquals('multi', $another_config->one->two->three);

    }

    function testErrorLeadingDot()
    {
        try {
            $config = new Akrabat_Config_Ini($this->_iniFileConfig, 'leadingdot');
        }
        catch (Exception $expected) {
            $this->assertContains('Invalid key', $expected->getMessage());
            return;
        }
        $this->fail('An expected Exception has not been raised.');
    }

    function testErrorOneDot()
    {
        try {
            $config = new Akrabat_Config_Ini($this->_iniFileConfig, 'onedot');
        }
        catch (Exception $expected) {
            $this->assertContains('Invalid key', $expected->getMessage());
            return;
        }
        $this->fail('An expected Exception has not been raised.');
    }

    function testErrorTwoDots()
    {
        try {
            $config = new Akrabat_Config_Ini($this->_iniFileConfig, 'twodots');
        }
        catch (Exception $expected) {
            $this->assertContains('Invalid key', $expected->getMessage());
            return;
        }
        $this->fail('An expected Exception has not been raised.');
    }

    function testErrorThreeDots()
    {
        try {
            $config = new Akrabat_Config_Ini($this->_iniFileConfig, 'threedots');
        }
        catch (Exception $expected) {
            $this->assertContains('Invalid key', $expected->getMessage());
            return;
        }
        $this->fail('An expected Exception has not been raised.');
    }

    function testErrorTrailingDot()
    {
        try {
            $config = new Akrabat_Config_Ini($this->_iniFileConfig, 'trailingdot');
        }
        catch (Exception $expected) {
            $this->assertContains('Invalid key', $expected->getMessage());
            return;
        }
        $this->fail('An expected Exception has not been raised.');
    }

    function testErrorNoInitialSection()
    {
        try {
            $current_error_reporting = error_reporting(0);
            $config = new Akrabat_Config_Ini($this->_iniFileConfig);
            error_reporting($current_error_reporting);
        }
        catch (Exception $expected) {
            $this->assertContains('Section is not set', $expected->getMessage());
            return;
        }

        $this->fail('An expected Exception has not been raised.');
    }

    function testErrorNoExtendsSection()
    {
        try {
            $config = new Akrabat_Config_Ini($this->_iniFileConfig, 'extendserror');
        }
        catch (Exception $expected) {
            $this->assertContains('cannot be found', $expected->getMessage());
            return;
        }

        $this->fail('An expected Exception has not been raised.');
    }

    function testErrorWriteToReadOnly()
    {
        $config = new Akrabat_Config_Ini($this->_iniFileConfig, 'all');
        try {
            $config->test = '32';
        }
        catch (Exception $expected) {
            $this->assertContains('read only', $expected->getMessage());
            return;
        }

        $this->fail('An expected Exception has not been raised.');
    }

    /*
    function testSecondFileOverwritesFirst()
    {
    $config = new Akrabat_Config_Ini();
    $config->load($this->_iniFileOne);
    $this->assertEquals('one', $config->akey);

    $config->load($this->_iniFileTwo);
    $this->assertEquals('two', $config->akey);

    $this->assertEquals('two', $config->db['hostname']);
    $this->assertEquals('thisDb', $config->db['database']);
    }

    function testNestedOverridesOneSection()
    {
    $config = new Akrabat_Config_Ini($this->_iniFileConfig, 'test');
    $this->assertEquals('test', $config->db['name']);
    $this->assertEquals('username', $config->db['user']);

    }

    function testNestedOverridesMultiSection()
    {
    $config = new Akrabat_Config_Ini($this->_iniFileConfig);
    $this->assertEquals('test', $config->test['db']['name']);
    $this->assertEquals('username', $config->test['db']['user']);

    }
    */
}

?>